import { path, pipe, split, curry, not, isArrayLike, is, reject, isEmpty, unapply, identity, prepend, equals, isNil, apply } from 'ramda'
import dynamic from './dynamicNode'
import isPath from './isPath'
import listPath from './listPath'
import Route from 'route-parser'

function isUriMatch(a, b) {
  return equals(a, b)
}

const testRoute = curry((uri, fn, xs) => {
  let testUri = xs[0]

  if (not(new Route(uri).match(testUri))) {
    return false
  }

  if (isNil(fn)) {
    return true
  }

  let result = apply(fn, xs)

  if (not(is(Boolean, result))) {
    throw new Error(`The result of evaluating ${fn.toString()} is ${result}. This should have been a Boolean`)
  }

  return result

})
/**
 * Route dyamic node that returns true or false
 * if the given route and its dependencies resolve
 *
 * If the provided route does not match the current route, then
 * the function is not called.
 *
 * @function
 * @memberof lib
 * @param {string} uriPath - The uriPath that the current route has to match
 * @param {array}? cursors - A list of cursors that get the current state which will then be checked in the provided function
 * @param {function}? fn - A function that will receive the current route and the data from the cursors and should decide if the route is matched or not.
 * @return {boolean} - Whether the route is matched or not
 * @example
 *
 * let tree = new Baobab({
 *   route: {
 *      uri: ['foo', 'baz']
 *   },
 *   bars: 2,
 *   isFooVisible: route('/foo/bar', ['/bars'], (route, bars) => {
 *      console.log('Function was called')
 *      return bars === 5
 *   })
 * })
 *
 * lib.cursorGet(tree, '/isFooVisible') // false
 *
 * tree.select('route', 'uri').set(['foo', 'bar'])
 * lib.cursorGet(tree, '/isFooVisible') // Function was called | false
 *
 * tree.select('bars').set(5)
 * lib.cursorGet(tree, '/isFooVisible') // Function was called | true
 *
 */
const route = (uri, cursors = [], fn) => {

  if (not(isNil(cursors)) && not(isArrayLike(cursors))) {
    throw 'Cursors must be an array'
  }

  if (not(isNil(fn)) && not(is(Function, fn))) {
    throw 'The last parameter must be a function'
  }

  cursors = prepend('/route', cursors)

  return dynamic(cursors, pipe(unapply(identity), testRoute(uri, fn)))
}

export default route
